@@ -4,6 +4,7 @@
 # Copyright (c) Saga Inc.
 # Distributed under the terms of the GPL License.
 from copy import copy
+from time import perf_counter
 from typing import Any, Dict, List, Optional, Set, Tuple
 import pandas as pd
 
@@ -29,10 +30,6 @@ class ExcelImportStepPerformer(StepPerformer):
     @classmethod
     def step_display_name(cls) -> str:
         return 'Imported Excel File'
-    
-    @classmethod
-    def step_event_type(cls) -> str:
-        return 'excel_import_edit'
 
     @classmethod
     def saturate(cls, prev_state: State, params: Dict[str, Any]) -> Dict[str, Any]:
@@ -49,7 +46,7 @@ class ExcelImportStepPerformer(StepPerformer):
         **params
     ) -> Tuple[State, Optional[Dict[str, Any]]]:
         # Create a new step
-        post_state = copy(prev_state)
+        post_state = prev_state.copy()
 
         read_excel_params = {
             'sheet_name': sheet_names,
@@ -60,7 +57,9 @@ class ExcelImportStepPerformer(StepPerformer):
         if not has_headers:
             read_excel_params['header'] = None
 
+        pandas_start_time = perf_counter()
         df_dictonary = pd.read_excel(file_name, **read_excel_params, engine='openpyxl') 
+        pandas_processing_time = perf_counter() - pandas_start_time
 
         for sheet_name, df in df_dictonary.items():
             post_state.add_df_to_state(
@@ -69,7 +68,9 @@ class ExcelImportStepPerformer(StepPerformer):
                 df_name=get_valid_dataframe_name(post_state.df_names, sheet_name),
             )
 
-        return post_state, {}
+        return post_state, {
+            'pandas_processing_time': pandas_processing_time
+        }
 
 
     @classmethod
