@@ -9,13 +9,15 @@ This file contains helpful functions and classes for testing operations.
 
 import json
 from functools import wraps
-from typing import Any, Dict, List, Union
+from typing import Any, Dict, List, Optional, Tuple, Union
+from mitosheet.step_performers.graph_steps.plotly_express_graphs import DO_NOT_CHANGE_PAPER_BGCOLOR_DEFAULT, DO_NOT_CHANGE_PLOT_BGCOLOR_DEFAULT, DO_NOT_CHANGE_TITLE_FONT_COLOR_DEFAULT
+from numpy import number
 
 import pandas as pd
 from mitosheet.mito_widget import MitoWidget, sheet
 from mitosheet.parser import parse_formula
 from mitosheet.transpiler.transpile import transpile
-from mitosheet.types import ColumnHeader, MultiLevelColumnHeader
+from mitosheet.types import ColumnHeader, ColumnID, GraphID, MultiLevelColumnHeader
 from mitosheet.utils import dfs_to_array_for_json, get_new_id
 
 
@@ -249,6 +251,29 @@ class MitoWidgetTestWrapper:
                 }
             }
         )
+    
+    @check_transpiled_code_after_call
+    def concat_sheets(
+            self, 
+            join: str,
+            ignore_index: bool,
+            sheet_indexes: int
+        ) -> bool:
+
+        return self.mito_widget.receive_message(
+            self.mito_widget,
+            {
+                'event': 'edit_event',
+                'id': get_new_id(),
+                'type': 'concat_edit',
+                'step_id': get_new_id(),
+                'params': {
+                    'join': join,
+                    'ignore_index': ignore_index,
+                    'sheet_indexes': sheet_indexes
+                }
+            }
+        )
 
     @check_transpiled_code_after_call
     def drop_duplicates(
@@ -726,6 +751,126 @@ class MitoWidgetTestWrapper:
         )
 
 
+    def generate_graph(
+        self, 
+        graph_id: str,
+        graph_type: str, 
+        sheet_index: number,
+        safety_filter_turned_on_by_user: bool,
+        xAxisColumnIDs: List[ColumnID],
+        yAxisColumnIDs: List[ColumnID],
+        height: str,
+        width: str,
+        title_title: Optional[str]=None,
+        title_visible: bool=True,
+        title_font_color: str=DO_NOT_CHANGE_TITLE_FONT_COLOR_DEFAULT,
+        xaxis_title: Optional[str]=None,
+        xaxis_visible: bool=True,
+        xaxis_title_font_color: str=DO_NOT_CHANGE_TITLE_FONT_COLOR_DEFAULT,
+        xaxis_rangeslider_visible: bool=True,
+        yaxis_title: Optional[str]=None,
+        yaxis_visible: bool=True,
+        yaxis_title_font_color: str=DO_NOT_CHANGE_TITLE_FONT_COLOR_DEFAULT,
+        showlegend: bool=True,
+        step_id: str=None,
+        paper_bgcolor: str=DO_NOT_CHANGE_PAPER_BGCOLOR_DEFAULT,
+        plot_bgcolor: str=DO_NOT_CHANGE_PLOT_BGCOLOR_DEFAULT,
+    ) -> bool:
+        return self.mito_widget.receive_message(
+            self.mito_widget,
+            {
+                'event': 'edit_event',
+                'id': get_new_id(),
+                'type': 'graph_edit',
+                'step_id': get_new_id() if step_id is None else step_id,
+                'params': {
+                    'graph_id': graph_id,
+                    'graph_preprocessing': {
+                        'safety_filter_turned_on_by_user': safety_filter_turned_on_by_user
+                    },
+                    'graph_creation': {
+                        'graph_type': graph_type,
+                        'sheet_index': sheet_index,
+                        'x_axis_column_ids': xAxisColumnIDs,
+                        'y_axis_column_ids': yAxisColumnIDs,
+                    },
+                    'graph_styling': {
+                        'title': {
+                            'title': title_title,
+                            'visible': title_visible,
+                            'title_font_color': title_font_color
+                        },
+                        'xaxis': {
+                            'title': xaxis_title,
+                            'visible': xaxis_visible,
+                            'title_font_color': xaxis_title_font_color,
+                            'rangeslider': {
+                                'visible': xaxis_rangeslider_visible
+                            }
+                        },
+                        'yaxis': {
+                            'title': yaxis_title,
+                            'visible': yaxis_visible,
+                            'title_font_color': yaxis_title_font_color,
+                        },
+                        'showlegend': showlegend,
+                        'paper_bgcolor': paper_bgcolor,
+                        'plot_bgcolor': plot_bgcolor,
+                    },
+                    'graph_rendering': {
+                        'height': height,
+                        'width': width
+                    }
+                }
+            }
+        )
+
+    def delete_graph(self, graph_id: GraphID) -> bool:
+        return self.mito_widget.receive_message(
+            self.mito_widget,
+            {
+                'event': 'edit_event',
+                'id': get_new_id(),
+                'type': 'graph_delete_edit',
+                'step_id': get_new_id(),
+                'params': {
+                    'graph_id': graph_id,
+                }
+            }
+        )
+
+    def duplicate_graph(self, old_graph_id: GraphID, new_graph_id: GraphID) -> bool:
+        return self.mito_widget.receive_message(
+            self.mito_widget,
+            {
+                'event': 'edit_event',
+                'id': get_new_id(),
+                'type': 'graph_duplicate_edit',
+                'step_id': get_new_id(),
+                'params': {
+                    'old_graph_id': old_graph_id,
+                    'new_graph_id': new_graph_id
+                }
+            }
+        )
+
+
+    def rename_graph(self, graph_id: GraphID, new_graph_tab_name: str) -> bool:
+        return self.mito_widget.receive_message(
+            self.mito_widget,
+            {
+                'event': 'edit_event',
+                'id': get_new_id(),
+                'type': 'graph_rename_edit',
+                'step_id': get_new_id(),
+                'params': {
+                    'graph_id': graph_id,
+                    'new_graph_tab_name': new_graph_tab_name
+                }
+            }
+        )
+
+
     def get_formula(self, sheet_index: int, column_header: ColumnHeader) -> str:
         """
         Gets the formula for a given column. Returns an empty
@@ -780,7 +925,59 @@ class MitoWidgetTestWrapper:
             return self.mito_widget.steps_manager.dfs[sheet_index][column_header].tolist()
         return self.mito_widget.steps_manager.dfs[sheet_index][column_header]
 
+    def get_graph_data(self, graph_id: str) -> Dict[str, Dict[str, Any]]: 
+        """
+        Returns the graph_data object 
+        """
+        if graph_id in self.mito_widget.steps_manager.curr_step.final_defined_state.graph_data_dict.keys():
+            return self.mito_widget.steps_manager.curr_step.final_defined_state.graph_data_dict[graph_id]
+        else:
+            return {}
+
+    def get_graph_type(self, graph_id: str) -> str:
+        """
+        Returns the graph type 
+        """
+        graph_data = self.get_graph_data(graph_id)
+        if bool(graph_data):
+            return graph_data["graphParams"]["graphCreation"]["graph_type"]
+        return ''
+
+    def get_graph_sheet_index(self, graph_id: str) -> int:
+        """
+        Returns the graph sheet index 
+        """
+        graph_data = self.get_graph_data(graph_id)
+        if bool(graph_data):
+            return graph_data["graphParams"]["graphCreation"]["sheet_index"]
+        return -1
+
+    def get_graph_axis_column_ids(self, graph_id: str, axis: str) -> List[str]:
+        """
+        Returns the graph axis column ids for either the x or y axis
+        """
+        graph_data = self.get_graph_data(graph_id)
+        if bool(graph_data):
+            if axis == 'x':
+                return graph_data["graphParams"]["graphCreation"]["x_axis_column_ids"]
+            if axis == 'y':
+                return graph_data["graphParams"]["graphCreation"]["y_axis_column_ids"]
+        return []
     
+    def get_is_graph_output_none(self, graph_id: str) -> bool:
+        """
+        Returns true if all of the graphOuput is does not exist.
+        """
+        return "graphOutput" not in self.get_graph_data(graph_id)
+
+    def get_graph_styling_params(self, graph_id: str) -> Dict[str, Optional[Union[str, bool]]]:
+        """
+        Returns the object that stores all the graph styling params, so that 
+        we can easily make sure the structure is correct
+        """
+        graph_data = self.get_graph_data(graph_id)
+        return graph_data["graphParams"]["graphStyling"]
+
 
 def create_mito_wrapper(sheet_one_A_data: List[Any], sheet_two_A_data: List[Any]=None) -> MitoWidgetTestWrapper:
     """
